package com.javahonk.solace.messaging;

import java.util.HashMap;
import java.util.Map;

import javax.annotation.PostConstruct;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;

import com.solacesystems.jcsmp.DeliveryMode;
import com.solacesystems.jcsmp.EndpointProperties;
import com.solacesystems.jcsmp.InvalidPropertiesException;
import com.solacesystems.jcsmp.JCSMPException;
import com.solacesystems.jcsmp.JCSMPFactory;
import com.solacesystems.jcsmp.JCSMPProperties;
import com.solacesystems.jcsmp.JCSMPSession;
import com.solacesystems.jcsmp.JCSMPStreamingPublishEventHandler;
import com.solacesystems.jcsmp.Queue;
import com.solacesystems.jcsmp.TextMessage;
import com.solacesystems.jcsmp.XMLMessageProducer;

public class SolaceOutputAdapter {
	
	private static final Logger logger = LogManager.getLogger(SolaceOutputAdapter.class.getName());
	
	private String host;
	private String vpn;
	private String userName;
	private String userPassword;
	private String solaceOutputQueueName;
	private String deliveryMode;
	private  XMLMessageProducer publisher;	
	private Queue queue;
	
	@Value("${SERVER_NUM_RETRIES}")
	private int serverNumRetries;
	
	@Value("${SERVER_RECONNECT_INTERVAL}")
	private int serverReconnectInterval;
	
	@Value("${LOG_DETAILS}")
	private boolean logDetails;
	
	private int retries = 0;
	
	public SolaceOutputAdapter(String host, String vpn, String userName,
			String userPassword, String solaceOutputQueueName,
			String deliveryMode) {
		this.host = host;
		this.vpn = vpn;
		this.userName = userName;
		this.userPassword = userPassword;
		this.solaceOutputQueueName = solaceOutputQueueName;
		this.deliveryMode = deliveryMode;
	}
	
	@PostConstruct
	public void init() {
		
		try {
			
			if (logDetails)
			logger.info("Connection info:\n Host: {} \n VPN: {}\n User Name: {}\n Password: {}\n Queue Name: {}\n Delivery Mode: {}\n",
					host, vpn, userName, userPassword, solaceOutputQueueName,
					deliveryMode);

			final EndpointProperties endpointProps = new EndpointProperties();	        
	        endpointProps.setPermission(EndpointProperties.PERMISSION_CONSUME);
	        endpointProps.setAccessType(EndpointProperties.ACCESSTYPE_EXCLUSIVE);
			queue = JCSMPFactory.onlyInstance().createQueue(" "+solaceOutputQueueName);	        
	        
			getSession().provision(queue, endpointProps, JCSMPSession.FLAG_IGNORE_ALREADY_EXISTS);
	        
	        publisher = getSession().getMessageProducer(
            new JCSMPStreamingPublishEventHandler() {
                public void responseReceived(String messageID) {
                	logger.info("Producer received response for msg ID {}",messageID);
                }
                public void handleError(String messageID, JCSMPException e, long timestamp) {
                	logger.info("Producer received error for msg ID {}", messageID,timestamp,e);
                }
            });
	        
		} catch (Exception e) {
			
			retries++;	
			logger.error("Number of retries to Solace: {}", retries);
			logger.error("Error creating connection to Solace:", e);						
			while (retries < serverNumRetries) {
				try {					
					Thread.sleep(serverReconnectInterval*1000);
					init();
				} catch (Exception e1) {
					logger.error("Error creating connection to Solace", e);
				}								
			}
		} 
	}
	
	public JCSMPSession getSession() {
		
		JCSMPSession session = null;
		
		try {
			
			JCSMPProperties properties = new JCSMPProperties();
	        properties.setProperty(JCSMPProperties.HOST, host);
	        properties.setProperty(JCSMPProperties.VPN_NAME, vpn);
	        properties.setProperty(JCSMPProperties.USERNAME, userName);
	        properties.setProperty(JCSMPProperties.PASSWORD, userPassword);
	        session = JCSMPFactory.onlyInstance().createSession(properties);
	        
		} catch (InvalidPropertiesException e) {
			logger.error("Failed to create Solace session", e);
		} 
		
		return session;		
	}
	
	public TextMessage getTextMessage(String message) {
		
		TextMessage msg = JCSMPFactory.onlyInstance().createMessage(TextMessage.class);
		if (deliveryMode.trim().equalsIgnoreCase("NON_PERSISTENT")) {
			msg.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
		}else {
			msg.setDeliveryMode(DeliveryMode.PERSISTENT);
		}        
        msg.setText(message);
        
        return msg;
	}
	
	public Queue getQueue() {
		
		return queue;		
	}
	
	public XMLMessageProducer getPublisher() {
		
		return publisher;		
	}
	
	public void sendMessage(String message) {
		
		try {
			
			logger.info("Sending message: {}", message);
			getPublisher().send(getTextMessage(message), getQueue());
			
		} catch (JCSMPException e) {
			logger.error("Failed to send message", e);
		}	
	}
	
	public boolean isConnected() {
		
		return !getSession().isClosed();		
	}
	
	public Map<String, String> getConnectionInfo() {
		
		Map<String, String> connectionInfo = new HashMap<String, String>();
		
		connectionInfo.put("host", host);
		connectionInfo.put("vpn", vpn);
		connectionInfo.put("userName", userName);
		connectionInfo.put("solaceOutputQueueName", solaceOutputQueueName);
		connectionInfo.put("deliveryMode", deliveryMode);
		
		return connectionInfo;		
	}
	
	

}
